@@ -8,6 +8,9 @@ import kvstore
 
 PREFIX = 'instances'
 TMPLPREFIX = 'templates'
+# Characters used to replace slash in IDs
+SLASH = '--'
+DOT = '__'
 # Create a global kvstore client
 ENDPOINT = 'http://10.112.0.101:8500/v1/kv'
 #ENDPOINT = 'http://127.0.0.1:8500/v1/kv'
@@ -29,6 +32,7 @@ class UnsupportedTypeError(Exception):
 class UnsupportedTemplateFormatError(Exception):
     pass
 
+
 class KeyDoesNotExist(Exception):
     pass
 
@@ -57,7 +61,6 @@ def register(name, version, description,
     return Template(dn)
 
 
-
 def get_services():
     """Get the current list of registered services"""
     subtree = _kv.recurse(TMPLPREFIX)
@@ -81,6 +84,7 @@ def deregister(name, version):
     dn = '{}/{}/{}'.format(TMPLPREFIX, name, version)
     _kv.delete(dn, recursive=True)
 
+
 def instantiate(user=None, framework=None, flavour=None, options=None):
     """Register a new instance using information from the service template"""
     service = get_service_template(framework, flavour)
@@ -110,17 +114,15 @@ def instantiate(user=None, framework=None, flavour=None, options=None):
     else:
         raise UnsupportedTemplateFormatError('type: {}'.format(service.templatetype))
 
-    #import pprint
-    #pprint.pprint(data)
     kvinfo = {}
     _populate(kvinfo, using=data, prefix=dn)
     save(kvinfo)
     return Cluster(dn)
 
 
-def deinstantiate(user, framework, flavour):
+def deinstantiate(user, framework, flavour, instanceid):
     """Deinstantiate (remove) a given cluster instance"""
-    dn = '{}/{}/{}/{}'.format(PREFIX, user, framework, flavour)
+    dn = '{}/{}/{}/{}/{}'.format(PREFIX, user, framework, flavour, instanceid)
     _kv.delete(dn, recursive=True)
 
 
@@ -131,11 +133,13 @@ def save(kvinfo):
 
 
 def _populate(result, using, prefix=''):
-    """Converts a data dict in a flat key:value data structure"""
-    # naming the input data as using is just a convenient way to
-    # define the clear intent when calling this function that
-    # has the weird behaviour of returning the result inside one
-    # of the arguments
+    """Converts a data dict in a flat key:value data structure
+
+       Naming the input data as using is just a convenient way to
+       define the clear intent when calling this function that
+       has the weird behaviour of returning the result inside one
+       of the arguments
+    """
     data = using
 
     if isvalue(data):
@@ -222,14 +226,14 @@ def _merge(options):
 
 def _obstain_dns(user=None, service=None, version=None):
     """ Get all the dns using parameters as filters (e.g.: gluster instances of a user)"""
-    dn = '{}'.format(PREFIX)
-    if user is not None:
-        dn = '{}/{}'.format(dn, user)
+    # FIXME not finished
+    if user:
+        dn = '{}/{}'.format(PREFIX, user)
 
-    if service is not None:
+    if service:
         dn = '{}/{}'.format(dn, service)
 
-    if version is not None:
+    if version:
         dn = '{}/{}'.format(dn, version)
 
     # FIXME this may not escalate with hundreds of instances
@@ -494,6 +498,12 @@ class Node(object):
         value = ','.join(check_ports)
         _kv.set(dn, value)
 
+    @property
+    def cluster(self):
+        """Contains the cluster instance to which this node belgons to"""
+        clusterdn = extract_clusterdn_from_nodedn(self._endpoint)
+        return Cluster(clusterdn)
+
     def __str__(self):
         return str(self._endpoint)
 
@@ -518,6 +528,12 @@ class Node(object):
         }
 
 
+def extract_clusterdn_from_nodedn(nodedn):
+    """Extract the cluster DN from the given node DN"""
+    m = re.search(r'^(.*)/nodes/[^/]+$', nodedn)
+    return m.group(1)
+
+
 class Service(object):
     """Represents a service"""
     def __init__(self, endpoint):
@@ -712,3 +728,21 @@ def _parse_service_name(endpoint):
     """Parse the service name part of a given endpoint"""
     m = re.match(r'^{}/([^/]+)'.format(TMPLPREFIX), endpoint)
     return m.group(1)
+
+
+def id_from(dn):
+    """Convert a DN string in an ID string
+
+    Basically the ID string is equivalent to a DN but without
+    certain characters that can cause problems like '/'
+    """
+    return dn.replace('/', SLASH).replace('.', DOT)
+
+
+def dn_from(id):
+    """Convert an ID string into a DN string
+
+    Basically the ID string is equivalent to a DN but without
+    certain characters that can cause problems like '/'
+    """
+    return id.replace('.', DOT).replace(SLASH, '/')
